/* life.h */

/*
 * Copyright (c) 1997-2002 Hoshi Takanori
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

/*
 * types and macros
 */

typedef long cell_t;
typedef long offset_t;
typedef short coord_t;
typedef long gen_t;

#define SENTINEL ((cell_t) 0x7fffffff)
#define BAD_CELL ((cell_t) 0x80008000)

#define CELL(x, y) ((((cell_t) (x) + (1 << 14)) <<  0) \
		  + (((cell_t) (y) + (1 << 14)) << 16))
#define GET_X(cell) ((coord_t) ((cell) >>  0) - (1 << 14))
#define GET_Y(cell) ((coord_t) ((cell) >> 16) - (1 << 14))

#define ADJUST(x, y) ((cell_t) (x) + ((cell_t) (y) << 16))

#define CREATOR 'LIFE'
#define OLD_PREFS_VERSION 1
#define PREFS_VERSION 2
#define VFS_NAME_LENGTH 16


/*
 * global variables
 */

extern UInt32 rom_version;

struct old_prefs {
    int mode;

    coord_t scroll_x;
    coord_t scroll_y;
    int magnify;

    int last_top;
    int last_selection;
    LocalID last_id;

    int rand_type;
    coord_t rand_width;
    coord_t rand_height;
    int rand_density;
    coord_t rand_radius;
    int rand_density1;
    int rand_density2;

    int reserved;
};

struct prefs {
    int mode;

    coord_t scroll_x;
    coord_t scroll_y;
    int magnify;

    int last_top;
    int last_selection;
    LocalID last_id;

    int rand_type;
    coord_t rand_width;
    coord_t rand_height;
    int rand_density;
    coord_t rand_radius;
    int rand_density1;
    int rand_density2;

    int reserved;

    int vfs_top;
    int vfs_selection;
    char vfs_name[VFS_NAME_LENGTH];
};

extern struct prefs prefs;

#ifdef USE_HANDERA_VGA
Boolean handera_vga;
#endif


/*
 * function prototypes
 */

/* from life.c */

void reset_life(offset_t length);
offset_t count_length(const cell_t *begin, const cell_t *end);

const cell_t *incr_ptr(const cell_t *ptr);
const cell_t *decr_ptr(const cell_t *ptr);

/* from db.c */

void read_db(cell_t *tape, offset_t tape_length,
    offset_t *init_length, gen_t *generation, offset_t *cur_length);

void write_db(const cell_t *tape, offset_t init_length,
    gen_t generation, const cell_t *life_begin, const cell_t *life_end);

/* from load.c */

void load_memo(cell_t *tape, offset_t tape_length);
void load_vfs(cell_t *tape, offset_t tape_length);

/* from merge.c */

cell_t *merge(
    const cell_t *tape_begin, const cell_t *tape_end,
    const cell_t *life_begin, const cell_t *life_end,
    cell_t *out_begin, const cell_t *out_max);

/* from check.c */

int check(const cell_t *life_begin, const cell_t *life_end, int *num_gliders);

void get_bounds(const cell_t *life_begin, const cell_t *life_end,
    coord_t *min_x, coord_t *min_y, coord_t *max_x, coord_t *max_y);

/* from rand.c */

void random_life(cell_t *tape, offset_t tape_length);

/* from util.c */

void *GetObjectPtrIn(FormPtr form, UInt16 id);
void *GetObjectPtr(UInt16 id);
void GetObjectBounds(UInt16 id, RectanglePtr bounds);
void ShowHideObject(FormPtr form, UInt16 id, Boolean show);
void SetFieldNumber(FieldPtr field, Char *str, long number);
void SetFieldNumHandle(FormPtr form, UInt16 id, long number);
long GetFieldNumber(FormPtr form, UInt16 id);
UInt16 FindCategory(DmOpenRef db_ref, const Char *category_name);
Boolean IsMenuVisible(void);
Int16 GetTopItem(ListPtr list);
